#include "clock.h"

/* Diese Funktion ermittelt die Anzahl aktuell abgelaufener
 * Taktzyklen aus dem Performance-Counter Register der x86
 * Architektur.
 *
 * Dieses Register ermöglicht die Messung von sehr kleinen
 * Zeitdifferenzen, bei denen die C-Funktion "clock()", die 
 * sonst die portablere und damit zu bevorzugende Möglichkeit 
 * wäre, keine Zeitdifferenz feststellen könnte.
 * */
uint64_t x86clock()
{
  union {
    uint64_t int64;
    struct {
      uint32_t lo, hi;
    } int32;
  } counter;

  asm volatile (
    "rdtsc\n"
    "movl %%eax, %0\n"
    "movl %%edx, %1\n"
    : "=r" (counter.int32.lo), "=r" (counter.int32.hi)
    : 
    : "eax", "edx");
  return counter.int64;
}
